@@ -32,8 +32,8 @@
  * can't be locked.  Thus, this must always be called in a while loop to stabilize
  * things, since we can always lose the race condition getting to the parent vnode.
  */
-int cm_HaveAccessRights(struct cm_scache *scp, struct cm_user *up, long rights,
-                        long *outRightsp)
+int cm_HaveAccessRights(struct cm_scache *scp, struct cm_user *up, afs_uint32 rights,
+                        afs_uint32 *outRightsp)
 {
     cm_scache_t *aclScp;
     long code;
@@ -119,7 +119,6 @@ long cm_GetAccessRights(struct cm_scache *scp, struct cm_user *up,
     long code;
     cm_fid_t tfid;
     cm_scache_t *aclScp;
-    int got_cb = 0;
 
     /* pretty easy: just force a pass through the fetch status code */
         
@@ -128,14 +127,10 @@ long cm_GetAccessRights(struct cm_scache *scp, struct cm_user *up,
     /* first, start by finding out whether we have a directory or something
      * else, so we can find what object's ACL we need.
      */
-    if (!cm_HaveCallback(scp)) {
-	code = cm_SyncOp(scp, NULL, up, reqp, 0, 
-			 CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
-	if (code) 
-	    return code;
-
-	got_cb = 1;
-    }
+    code = cm_SyncOp(scp, NULL, up, reqp, 0, CM_SCACHESYNC_GETSTATUS
+                      | CM_SCACHESYNC_NEEDCALLBACK);
+    if (code) 
+        return code;
         
     if (scp->fileType != CM_SCACHETYPE_DIRECTORY) {
         /* not a dir, use parent dir's acl */
@@ -149,14 +144,15 @@ long cm_GetAccessRights(struct cm_scache *scp, struct cm_user *up,
             lock_ObtainMutex(&scp->mx);
             return code;
         }       
-        osi_Log1(afsd_logp, "GetAccess parent scp %x user %x", aclScp, up);
-	lock_ObtainMutex(&aclScp->mx);
-	code = cm_GetCallback(aclScp, up, reqp, 1);
-	lock_ReleaseMutex(&aclScp->mx);
+                
+        osi_Log1(afsd_logp, "GetAccess parent %x", aclScp);
+        lock_ObtainMutex(&aclScp->mx);
+        code = cm_GetCallback(aclScp, up, reqp, 1);
+        lock_ReleaseMutex(&aclScp->mx);
         cm_ReleaseSCache(aclScp);
         lock_ObtainMutex(&scp->mx);
-    } else if (!got_cb) {
-	code = cm_GetCallback(scp, up, reqp, 1);
+    } else {
+        code = cm_GetCallback(scp, up, reqp, 1);
     }
 
     return code;
