@@ -46,147 +46,22 @@ void cm_AdjustLRU(cm_scache_t *scp)
 {
     if (scp == cm_data.scacheLRULastp)
         cm_data.scacheLRULastp = (cm_scache_t *) osi_QPrev(&scp->q);
-    osi_QRemoveHT((osi_queue_t **) &cm_data.scacheLRUFirstp, (osi_queue_t **) &cm_data.scacheLRULastp, &scp->q);
+    osi_QRemove((osi_queue_t **) &cm_data.scacheLRUFirstp, &scp->q);
     osi_QAdd((osi_queue_t **) &cm_data.scacheLRUFirstp, &scp->q);
     if (!cm_data.scacheLRULastp) 
         cm_data.scacheLRULastp = scp;
 }
 
-/* called with cm_scacheLock write-locked; recycles an existing scp. */
-long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags)
-{
-    cm_scache_t **lscpp;
-    cm_scache_t *tscp;
-    int i;
-
-    if (scp->flags & CM_SCACHEFLAG_INHASH) {
-	/* hash it out first */
-	i = CM_SCACHE_HASH(&scp->fid);
-	for (lscpp = &cm_data.hashTablep[i], tscp = cm_data.hashTablep[i];
-	      tscp;
-	      lscpp = &tscp->nextp, tscp = tscp->nextp) {
-	    if (tscp == scp) {
-		*lscpp = scp->nextp;
-		scp->flags &= ~CM_SCACHEFLAG_INHASH;
-		break;
-	    }
-	}
-	osi_assertx(tscp, "afsd: scache hash screwup");
-    }
-
-    if (flags & CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS) {
-	osi_queueData_t *qdp;
-	cm_buf_t *bufp;
-
-	while(qdp = scp->bufWritesp) {
-            bufp = osi_GetQData(qdp);
-	    osi_QRemove((osi_queue_t **) &scp->bufWritesp, &qdp->q);
-	    osi_QDFree(qdp);
-	    if (bufp) {
-		lock_ObtainMutex(&bufp->mx);
-		bufp->cmFlags &= ~CM_BUF_CMSTORING;
-		bufp->flags &= ~CM_BUF_DIRTY;
-		bufp->dataVersion = -1; /* bad */
-		bufp->dirtyCounter++;
-		if (bufp->flags & CM_BUF_WAITING) {
-		    osi_Log2(afsd_logp, "CM RecycleSCache Waking [scp 0x%x] bufp 0x%x", scp, bufp);
-		    osi_Wakeup((long) &bufp);
-		}
-		lock_ReleaseMutex(&bufp->mx);
-		buf_Release(bufp);
-	    }
-        }
-	while(qdp = scp->bufReadsp) {
-            bufp = osi_GetQData(qdp);
-	    osi_QRemove((osi_queue_t **) &scp->bufReadsp, &qdp->q);
-	    osi_QDFree(qdp);
-	    if (bufp) {
-		lock_ObtainMutex(&bufp->mx);
-		bufp->cmFlags &= ~CM_BUF_CMFETCHING;
-		bufp->flags &= ~CM_BUF_DIRTY;
-		bufp->dataVersion = -1; /* bad */
-		bufp->dirtyCounter++;
-		if (bufp->flags & CM_BUF_WAITING) {
-		    osi_Log2(afsd_logp, "CM RecycleSCache Waking [scp 0x%x] bufp 0x%x", scp, bufp);
-		    osi_Wakeup((long) &bufp);
-		}
-		lock_ReleaseMutex(&bufp->mx);
-		buf_Release(bufp);
-	    }
-        }
-    } else {
-	/* look for things that shouldn't still be set */
-	osi_assert(scp->bufWritesp == NULL);
-	osi_assert(scp->bufReadsp == NULL);
-    }
-
-    /* invalidate so next merge works fine;
-     * also initialize some flags */
-    scp->flags &= ~(CM_SCACHEFLAG_STATD
-		     | CM_SCACHEFLAG_RO
-		     | CM_SCACHEFLAG_PURERO
-		     | CM_SCACHEFLAG_OVERQUOTA
-		     | CM_SCACHEFLAG_OUTOFSPACE);
-    scp->serverModTime = 0;
-    scp->dataVersion = 0;
-    scp->bulkStatProgress = hzero;
-    scp->waitCount = 0;
-
-    scp->fid.vnode = 0;
-    scp->fid.volume = 0;
-    scp->fid.unique = 0;
-    scp->fid.cell = 0;
-
-    /* discard callback */
-    if (scp->cbServerp) {
-	cm_PutServer(scp->cbServerp);
-	scp->cbServerp = NULL;
-    }
-    scp->cbExpires = 0;
-
-    /* remove from dnlc */
-    cm_dnlcPurgedp(scp);
-    cm_dnlcPurgevp(scp);
-
-    /* discard cached status; if non-zero, Close
-     * tried to store this to server but failed */
-    scp->mask = 0;
-
-    /* drop held volume ref */
-    if (scp->volp) {
-	cm_PutVolume(scp->volp);
-	scp->volp = NULL;
-    }
-
-    /* discard symlink info */
-    scp->mountPointStringp[0] = 0;
-    memset(&scp->mountRootFid, 0, sizeof(cm_fid_t));
-    memset(&scp->dotdotFid, 0, sizeof(cm_fid_t));
-
-    /* reset locking info */
-    scp->fileLocksH = NULL;
-    scp->fileLocksT = NULL;
-    scp->serverLock = (-1);
-    scp->exclusiveLocks = 0;
-    scp->sharedLocks = 0;
-
-    /* not locked, but there can be no references to this guy
-     * while we hold the global refcount lock.
-     */
-    cm_FreeAllACLEnts(scp);
-
-    return 0;
-}
-
-
 /* called with cm_scacheLock write-locked; find a vnode to recycle.
  * Can allocate a new one if desperate, or if below quota (cm_data.maxSCaches).
  */
 cm_scache_t *cm_GetNewSCache(void)
 {
     cm_scache_t *scp;
+    int i;
+    cm_scache_t **lscpp;
+    cm_scache_t *tscp;
 
-  start:
     if (cm_data.currentSCaches >= cm_data.maxSCaches) {
         for (scp = cm_data.scacheLRULastp;
               scp;
@@ -197,21 +72,88 @@ cm_scache_t *cm_GetNewSCache(void)
                 
         if (scp) {
             osi_assert(scp >= cm_data.scacheBaseAddress && scp < (cm_scache_t *)cm_data.hashTablep);
+            /* we found an entry, so return it */
+            if (scp->flags & CM_SCACHEFLAG_INHASH) {
+                /* hash it out first */
+                i = CM_SCACHE_HASH(&scp->fid);
+                for (lscpp = &cm_data.hashTablep[i], tscp = cm_data.hashTablep[i];
+                      tscp;
+                      lscpp = &tscp->nextp, tscp = tscp->nextp) {
+                    if (tscp == scp) {
+                        *lscpp = scp->nextp;
+                        scp->flags &= ~CM_SCACHEFLAG_INHASH;
+                        break;
+                    }
+                }
+                osi_assertx(tscp, "afsd: scache hash screwup");
+            }
 
-	    if (!cm_RecycleSCache(scp, 0)) {
-	    
-		/* we found an entry, so return it */
-		/* now remove from the LRU queue and put it back at the
-		 * head of the LRU queue.
-		 */
-		cm_AdjustLRU(scp);
+            /* look for things that shouldn't still be set */
+            osi_assert(scp->bufWritesp == NULL);
+            osi_assert(scp->bufReadsp == NULL);
 
-		/* and we're done */
-		return scp;
-	    } else {
-		/* We don't like this entry, choose another one. */
-		goto start;
-	    }
+            /* invalidate so next merge works fine;
+            * also initialize some flags */
+            scp->flags &= ~(CM_SCACHEFLAG_STATD
+                             | CM_SCACHEFLAG_RO
+                             | CM_SCACHEFLAG_PURERO
+                             | CM_SCACHEFLAG_OVERQUOTA
+                             | CM_SCACHEFLAG_OUTOFSPACE);
+            scp->serverModTime = 0;
+            scp->dataVersion = 0;
+            scp->bulkStatProgress = hzero;
+            scp->waitCount = 0;
+
+            scp->fid.vnode = 0;
+            scp->fid.volume = 0;
+            scp->fid.unique = 0;
+            scp->fid.cell = 0;
+
+            /* discard callback */
+            if (scp->cbServerp) {
+                cm_PutServer(scp->cbServerp);
+                scp->cbServerp = NULL;
+            }
+            scp->cbExpires = 0;
+
+            /* remove from dnlc */
+            cm_dnlcPurgedp(scp);
+            cm_dnlcPurgevp(scp);
+
+            /* discard cached status; if non-zero, Close
+             * tried to store this to server but failed */
+            scp->mask = 0;
+
+            /* drop held volume ref */
+            if (scp->volp) {
+                cm_PutVolume(scp->volp);
+                scp->volp = NULL;
+            }
+
+            /* discard symlink info */
+            scp->mountPointStringp[0] = 0;
+            memset(&scp->mountRootFid, 0, sizeof(cm_fid_t));
+            memset(&scp->dotdotFid, 0, sizeof(cm_fid_t));
+
+            /* reset locking info */
+            scp->fileLocksH = NULL;
+            scp->fileLocksT = NULL;
+            scp->serverLock = (-1);
+            scp->exclusiveLocks = 0;
+            scp->sharedLocks = 0;
+
+            /* not locked, but there can be no references to this guy
+             * while we hold the global refcount lock.
+             */
+            cm_FreeAllACLEnts(scp);
+
+            /* now remove from the LRU queue and put it back at the
+             * head of the LRU queue.
+             */
+            cm_AdjustLRU(scp);
+
+            /* and we're done */
+            return scp;
         }
     }
         
@@ -550,7 +492,7 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
             scp->fileType = CM_SCACHETYPE_INVALID;
 
         lock_ObtainMutex(&cm_Freelance_Lock);
-        scp->length.LowPart = strlen(mp)+4;
+        scp->length.LowPart = (DWORD)strlen(mp)+4;
         strncpy(scp->mountPointStringp,mp,MOUNTPOINTLEN);
         scp->mountPointStringp[MOUNTPOINTLEN-1] = '\0';
         lock_ReleaseMutex(&cm_Freelance_Lock);
@@ -563,10 +505,10 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
         scp->parentVnode=0x1;
         scp->group=0;
         scp->dataVersion=cm_data.fakeDirVersion;
+        scp->lockDataVersion=-1; /* no lock yet */
 	lock_ReleaseMutex(&scp->mx);
         *outScpp = scp;
         lock_ReleaseWrite(&cm_scacheLock);
-        /*afsi_log("   getscache done");*/
         return 0;
     }
     // end of yj code
@@ -641,7 +583,6 @@ long cm_GetSCache(cm_fid_t *fidp, cm_scache_t **outScpp, cm_user_t *userp,
     return 0;
 }
 
-
 /* Returns a held reference to the scache's parent 
  * if it exists */
 cm_scache_t * cm_FindSCacheParent(cm_scache_t * scp)
@@ -657,15 +598,15 @@ cm_scache_t * cm_FindSCacheParent(cm_scache_t * scp)
     parent_fid.unique = scp->parentUnique;
 
     if (cm_FidCmp(&scp->fid, &parent_fid)) {
-        i = CM_SCACHE_HASH(&parent_fid);
-	for (pscp = cm_data.hashTablep[i]; pscp; pscp = pscp->nextp) {
-	    if (!cm_FidCmp(&pscp->fid, &parent_fid)) {
-	        cm_HoldSCacheNoLock(pscp);
-		break;
+	for (i=0; i<cm_data.hashTableSize; i++) {
+	    for (pscp = cm_data.hashTablep[i]; pscp; pscp = pscp->nextp) {
+		if (!cm_FidCmp(&pscp->fid, &parent_fid)) {
+		    cm_HoldSCacheNoLock(pscp);
+		    break;
+		}
 	    }
 	}
     }
-
     lock_ReleaseWrite(&cm_scacheLock);
 
     return pscp;
@@ -727,12 +668,12 @@ cm_scache_t * cm_FindSCacheParent(cm_scache_t * scp)
  * CM_SCACHESYNC_STOREDATA_EXCL and CM_SCACHEFLAG_DATASTORING.
  */
 long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
-               long rights, long flags)
+               afs_uint32 rights, afs_uint32 flags)
 {
     osi_queueData_t *qdp;
     long code;
     cm_buf_t *tbufp;
-    long outRights;
+    afs_uint32 outRights;
     int bufLocked;
 
     /* lookup this first */
@@ -773,7 +714,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING | CM_SCACHEFLAG_STORING
                                | CM_SCACHEFLAG_SIZESTORING | CM_SCACHEFLAG_GETCALLBACK)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING|STORING|SIZESTORING|GETCALLBACK want FETCHSTATUS", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING|STORING|SIZESTORING|GETCALLBACK want FETCHSTATUS", scp);
                 goto sleep;
             }
         }
@@ -784,11 +725,11 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING | CM_SCACHEFLAG_STORING |
                               CM_SCACHEFLAG_SIZESTORING | CM_SCACHEFLAG_GETCALLBACK)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING|STORING|SIZESTORING|GETCALLBACK want STORESIZE|STORESTATUS|SETSIZE|GETCALLBACK", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING|STORING|SIZESTORING|GETCALLBACK want STORESIZE|STORESTATUS|SETSIZE|GETCALLBACK", scp);
                 goto sleep;
             }
             if (scp->bufReadsp || scp->bufWritesp) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is bufRead|bufWrite want STORESIZE|STORESTATUS|SETSIZE|GETCALLBACK", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is bufRead|bufWrite want STORESIZE|STORESTATUS|SETSIZE|GETCALLBACK", scp);
                 goto sleep;
             }
         }
@@ -799,11 +740,11 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING | CM_SCACHEFLAG_STORING
                                | CM_SCACHEFLAG_SIZESTORING | CM_SCACHEFLAG_GETCALLBACK)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING|STORING|SIZESTORING|GETCALLBACK want FETCHDATA", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING|STORING|SIZESTORING|GETCALLBACK want FETCHDATA", scp);
                 goto sleep;
             }
             if (bufp && (bufp->cmFlags & (CM_BUF_CMFETCHING | CM_BUF_CMSTORING))) {
-                osi_Log2(afsd_logp, "CM SyncOp scp 0x%x bufp 0x%x is BUF_CMFETCHING|BUF_CMSTORING want FETCHDATA", scp, bufp);
+                osi_Log2(afsd_logp, "CM SyncOp scp 0x%p bufp 0x%p is BUF_CMFETCHING|BUF_CMSTORING want FETCHDATA", scp, bufp);
                 goto sleep;
             }
         }
@@ -811,11 +752,11 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
             /* same as fetch data */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING | CM_SCACHEFLAG_STORING
                                | CM_SCACHEFLAG_SIZESTORING | CM_SCACHEFLAG_GETCALLBACK)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING|STORING|SIZESTORING|GETCALLBACK want STOREDATA", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING|STORING|SIZESTORING|GETCALLBACK want STOREDATA", scp);
                 goto sleep;
             }
             if (bufp && (bufp->cmFlags & (CM_BUF_CMFETCHING | CM_BUF_CMSTORING))) {
-                osi_Log2(afsd_logp, "CM SyncOp scp 0x%x bufp 0x%x is BUF_CMFETCHING|BUF_CMSTORING want STOREDATA", scp, bufp);
+                osi_Log2(afsd_logp, "CM SyncOp scp 0x%p bufp 0x%p is BUF_CMFETCHING|BUF_CMSTORING want STOREDATA", scp, bufp);
                 goto sleep;
             }
         }
@@ -823,7 +764,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
         if (flags & CM_SCACHESYNC_STOREDATA_EXCL) {
             /* Don't allow concurrent StoreData RPC's */
             if (scp->flags & CM_SCACHEFLAG_DATASTORING) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is DATASTORING want STOREDATA_EXCL", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is DATASTORING want STOREDATA_EXCL", scp);
                 goto sleep;
             }
         }
@@ -831,7 +772,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
         if (flags & CM_SCACHESYNC_ASYNCSTORE) {
             /* Don't allow more than one BKG store request */
             if (scp->flags & CM_SCACHEFLAG_ASYNCSTORING) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is ASYNCSTORING want ASYNCSTORE", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is ASYNCSTORING want ASYNCSTORE", scp);
                 goto sleep;
             }
         }
@@ -839,7 +780,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
         if (flags & CM_SCACHESYNC_LOCK) {
             /* Don't allow concurrent fiddling with lock lists */
             if (scp->flags & CM_SCACHEFLAG_LOCKING) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is LOCKING want LOCK", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is LOCKING want LOCK", scp);
                 goto sleep;
             }
         }
@@ -850,7 +791,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              * bringing in new status.
              */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING want GETSTATUS", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING want GETSTATUS", scp);
                 goto sleep;
             }
         }
@@ -864,7 +805,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              * changing here.
              */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING | CM_SCACHEFLAG_STORING | CM_SCACHEFLAG_SIZESTORING)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING|STORING|SIZESTORING want SETSTATUS", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING|STORING|SIZESTORING want SETSTATUS", scp);
                 goto sleep;
             }
         }
@@ -874,11 +815,11 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              * is OK to read while storing the data back.
              */
             if (scp->flags & CM_SCACHEFLAG_FETCHING) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING want READ", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING want READ", scp);
                 goto sleep;
             }
             if (bufp && ((bufp->cmFlags & (CM_BUF_CMFETCHING | CM_BUF_CMFULLYFETCHED)) == CM_BUF_CMFETCHING)) {
-                osi_Log2(afsd_logp, "CM SyncOp scp 0x%x bufp 0x%x is BUF_CMFETCHING want READ", scp, bufp);
+                osi_Log2(afsd_logp, "CM SyncOp scp 0x%p bufp 0x%p is BUF_CMFETCHING want READ", scp, bufp);
                 goto sleep;
             }
         }
@@ -888,11 +829,11 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
              */
             if (scp->flags & (CM_SCACHEFLAG_FETCHING | CM_SCACHEFLAG_STORING
                                | CM_SCACHEFLAG_SIZESTORING)) {
-                osi_Log1(afsd_logp, "CM SyncOp scp 0x%x is FETCHING|STORING|SIZESTORING want WRITE", scp);
+                osi_Log1(afsd_logp, "CM SyncOp scp 0x%p is FETCHING|STORING|SIZESTORING want WRITE", scp);
                 goto sleep;
             }
             if (bufp && (bufp->cmFlags & (CM_BUF_CMFETCHING | CM_BUF_CMSTORING))) {
-                osi_Log2(afsd_logp, "CM SyncOp scp 0x%x bufp 0x%x is BUF_CMFETCHING|BUF_CMSTORING want WRITE", scp, bufp);
+                osi_Log2(afsd_logp, "CM SyncOp scp 0x%p bufp 0x%p is BUF_CMFETCHING|BUF_CMSTORING want WRITE", scp, bufp);
                 goto sleep;
             }
         }
@@ -910,10 +851,9 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
 #endif /* AFS_FREELANCE_CLIENT */
              ) {
             if (!cm_HaveCallback(scp)) {
-                osi_Log1(afsd_logp, "CM SyncOp getting callback on scp %x",
-                          (long) scp);
-                if (bufLocked) 
-		    lock_ReleaseMutex(&bufp->mx);
+                osi_Log1(afsd_logp, "CM SyncOp getting callback on scp 0x%p",
+                          scp);
+                if (bufLocked) lock_ReleaseMutex(&bufp->mx);
                 code = cm_GetCallback(scp, up, reqp, 0);
                 if (bufLocked) {
                     lock_ReleaseMutex(&scp->mx);
@@ -934,19 +874,20 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
                 return CM_ERROR_READONLY;
 
             if (cm_HaveAccessRights(scp, up, rights, &outRights)) {
-                if (~outRights & rights) return CM_ERROR_NOACCESS;
+                if (~outRights & rights) 
+		    return CM_ERROR_NOACCESS;
             }
             else {
                 /* we don't know the required access rights */
                 if (bufLocked) lock_ReleaseMutex(&bufp->mx);
                 code = cm_GetAccessRights(scp, up, reqp);
+                if (code) 
+                    return code;
                 if (bufLocked) {
                     lock_ReleaseMutex(&scp->mx);
                     lock_ObtainMutex(&bufp->mx);
                     lock_ObtainMutex(&scp->mx);
                 }
-                if (code) 
-                    return code;
                 continue;
             }
         }
@@ -962,31 +903,28 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
             return CM_ERROR_WOULDBLOCK;
 
         /* wait here, then try again */
-        osi_Log1(afsd_logp, "CM SyncOp sleeping scp 0x%x", scp);
+        osi_Log1(afsd_logp, "CM SyncOp sleeping scp 0x%p", scp);
         if ( scp->flags & CM_SCACHEFLAG_WAITING ) {
             scp->waitCount++;
             scp->waitRequests++;
-            osi_Log3(afsd_logp, "CM SyncOp CM_SCACHEFLAG_WAITING already set for 0x%x; %d threads; %d requests", 
+            osi_Log3(afsd_logp, "CM SyncOp CM_SCACHEFLAG_WAITING already set for 0x%p; %d threads; %d requests", 
                      scp, scp->waitCount, scp->waitRequests);
         } else {
-            osi_Log1(afsd_logp, "CM SyncOp CM_SCACHEFLAG_WAITING set for 0x%x", scp);
+            osi_Log1(afsd_logp, "CM SyncOp CM_SCACHEFLAG_WAITING set for 0x%p", scp);
             scp->flags |= CM_SCACHEFLAG_WAITING;
             scp->waitCount = scp->waitRequests = 1;
         }
         if (bufLocked) 
             lock_ReleaseMutex(&bufp->mx);
-        osi_SleepM((long) &scp->flags, &scp->mx);
-
-	smb_UpdateServerPriority();
-
+        osi_SleepM((LONG_PTR) &scp->flags, &scp->mx);
         if (bufLocked) 
             lock_ObtainMutex(&bufp->mx);
         lock_ObtainMutex(&scp->mx);
         scp->waitCount--;
-        osi_Log3(afsd_logp, "CM SyncOp woke! scp 0x%x; still waiting %d threads of %d requests", 
+        osi_Log3(afsd_logp, "CM SyncOp woke! scp 0x%p; still waiting %d threads of %d requests", 
                  scp, scp->waitCount, scp->waitRequests);
         if (scp->waitCount == 0) {
-            osi_Log1(afsd_logp, "CM SyncOp CM_SCACHEFLAG_WAITING reset for 0x%x", scp);
+            osi_Log1(afsd_logp, "CM SyncOp CM_SCACHEFLAG_WAITING reset for 0x%p", scp);
             scp->flags &= ~CM_SCACHEFLAG_WAITING;
             scp->waitRequests = 0;
         }
@@ -1053,7 +991,7 @@ long cm_SyncOp(cm_scache_t *scp, cm_buf_t *bufp, cm_user_t *up, cm_req_t *reqp,
 /* for those syncops that setup for RPCs.
  * Called with scache locked.
  */
-void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, long flags)
+void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, afs_uint32 flags)
 {
     osi_queueData_t *qdp;
     cm_buf_t *tbufp;
@@ -1079,18 +1017,17 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, long flags)
         /* ensure that the buffer isn't already in the I/O list */
         for(qdp = scp->bufReadsp; qdp; qdp = (osi_queueData_t *) osi_QNext(&qdp->q)) {
             tbufp = osi_GetQData(qdp);
-            if (tbufp == bufp) 
-		break;
+            if (tbufp == bufp) break;
         }
-	if (qdp) {
-	    osi_QRemove((osi_queue_t **) &scp->bufReadsp, &qdp->q);
-	    osi_QDFree(qdp);
-	}
+        osi_assert(qdp != NULL);
+        osi_assert(osi_GetQData(qdp) == bufp);
+        osi_QRemove((osi_queue_t **) &scp->bufReadsp, &qdp->q);
+        osi_QDFree(qdp);
         if (bufp) {
             bufp->cmFlags &= ~(CM_BUF_CMFETCHING | CM_BUF_CMFULLYFETCHED);
             if (bufp->flags & CM_BUF_WAITING) {
-                osi_Log2(afsd_logp, "CM SyncOpDone Waking [scp 0x%x] bufp 0x%x", scp, bufp);
-                osi_Wakeup((long) &bufp);
+                osi_Log2(afsd_logp, "CM SyncOpDone Waking [scp 0x%p] bufp 0x%p", scp, bufp);
+                osi_Wakeup((LONG_PTR) &bufp);
             }
             buf_Release(bufp);
         }
@@ -1101,18 +1038,17 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, long flags)
         /* ensure that the buffer isn't already in the I/O list */
         for(qdp = scp->bufWritesp; qdp; qdp = (osi_queueData_t *) osi_QNext(&qdp->q)) {
             tbufp = osi_GetQData(qdp);
-            if (tbufp == bufp) 
-		break;
+            if (tbufp == bufp) break;
         }
-	if (qdp) {
-	    osi_QRemove((osi_queue_t **) &scp->bufWritesp, &qdp->q);
-	    osi_QDFree(qdp);
-	}
+        osi_assert(qdp != NULL);
+        osi_assert(osi_GetQData(qdp) == bufp);
+        osi_QRemove((osi_queue_t **) &scp->bufWritesp, &qdp->q);
+        osi_QDFree(qdp);
         if (bufp) {
             bufp->cmFlags &= ~CM_BUF_CMSTORING;
             if (bufp->flags & CM_BUF_WAITING) {
-                osi_Log2(afsd_logp, "CM SyncOpDone Waking [scp 0x%x] bufp 0x%x", scp, bufp);
-                osi_Wakeup((long) &bufp);
+                osi_Log2(afsd_logp, "CM SyncOpDone Waking [scp 0x%p] bufp 0x%p", scp, bufp);
+                osi_Wakeup((LONG_PTR) &bufp);
             }
             buf_Release(bufp);
         }
@@ -1120,8 +1056,8 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, long flags)
 
     /* and wakeup anyone who is waiting */
     if (scp->flags & CM_SCACHEFLAG_WAITING) {
-        osi_Log1(afsd_logp, "CM SyncOpDone Waking scp 0x%x", scp);
-        osi_Wakeup((long) &scp->flags);
+        osi_Log1(afsd_logp, "CM SyncOpDone Waking scp 0x%p", scp);
+        osi_Wakeup((LONG_PTR) &scp->flags);
     }
 }       
 
@@ -1139,7 +1075,7 @@ void cm_SyncOpDone(cm_scache_t *scp, cm_buf_t *bufp, long flags)
  * started before that, can cause old info to be merged from the first call.
  */
 void cm_MergeStatus(cm_scache_t *scp, AFSFetchStatus *statusp, AFSVolSync *volp,
-                    cm_user_t *userp, int flags)
+                    cm_user_t *userp, afs_uint32 flags)
 {
     // yj: i want to create some fake status for the /afs directory and the
     // entries under that directory
@@ -1362,7 +1298,7 @@ int cm_DumpSCache(FILE *outputFile, char *cookie, int lock)
         lock_ObtainRead(&cm_scacheLock);
   
     sprintf(output, "%s - dumping scache - cm_data.currentSCaches=%d, cm_data.maxSCaches=%d\n", cookie, cm_data.currentSCaches, cm_data.maxSCaches);
-    WriteFile(outputFile, output, strlen(output), &zilch, NULL);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
   
     for (scp = cm_data.scacheLRULastp; scp; scp = (cm_scache_t *) osi_QPrev(&scp->q)) 
     {
@@ -1371,12 +1307,12 @@ int cm_DumpSCache(FILE *outputFile, char *cookie, int lock)
             sprintf(output, "%s fid (cell=%d, volume=%d, vnode=%d, unique=%d) refCount=%u\n", 
                     cookie, scp->fid.cell, scp->fid.volume, scp->fid.vnode, scp->fid.unique, 
                     scp->refCount);
-            WriteFile(outputFile, output, strlen(output), &zilch, NULL);
+            WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
         }
     }
   
     sprintf(output, "%s - dumping cm_data.hashTable - cm_data.hashTableSize=%d\n", cookie, cm_data.hashTableSize);
-    WriteFile(outputFile, output, strlen(output), &zilch, NULL);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
   
     for (i = 0; i < cm_data.hashTableSize; i++)
     {
@@ -1384,16 +1320,16 @@ int cm_DumpSCache(FILE *outputFile, char *cookie, int lock)
         {
             if (scp->refCount != 0)
             {
-                sprintf(output, "%s scp=0x%08X, hash=%d, fid (cell=%d, volume=%d, vnode=%d, unique=%d) refCount=%u\n", 
-                         cookie, (void *)scp, i, scp->fid.cell, scp->fid.volume, scp->fid.vnode, 
+                sprintf(output, "%s scp=0x%p, hash=%d, fid (cell=%d, volume=%d, vnode=%d, unique=%d) refCount=%u\n", 
+                         cookie, scp, i, scp->fid.cell, scp->fid.volume, scp->fid.vnode, 
                          scp->fid.unique, scp->refCount);
-                WriteFile(outputFile, output, strlen(output), &zilch, NULL);
+                WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
             }
         }
     }
 
     sprintf(output, "%s - Done dumping scache.\n", cookie);
-    WriteFile(outputFile, output, strlen(output), &zilch, NULL);
+    WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
   
     if (lock)
         lock_ReleaseRead(&cm_scacheLock);       
