@@ -16,7 +16,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vfsops.c,v 1.29.2.23 2007/02/09 01:25:48 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_vfsops.c,v 1.42 2005/11/29 03:09:24 shadow Exp $");
 
 #define __NO_VERSION__		/* don't define kernel_version in module.h */
 #include <linux/module.h> /* early to avoid printf->printk mapping */
@@ -49,6 +49,7 @@ static void iattr2vattr(struct vattr *vattrp, struct iattr *iattrp);
 static int afs_root(struct super_block *afsp);
 struct super_block *afs_read_super(struct super_block *sb, void *data, int silent);
 int afs_fill_super(struct super_block *sb, void *data, int silent);
+static struct super_block *afs_get_sb(struct file_system_type *fs_type, int flags, const char *dev_name, void *data);
 
 /* afs_file_system
  * VFS entry for Linux - installed in init_module
@@ -57,30 +58,11 @@ int afs_fill_super(struct super_block *sb, void *data, int silent);
  * 2) Mount call comes to us via do_mount -> read_super -> afs_read_super.
  *    We are expected to setup the super_block. See afs_read_super.
  */
-
-
-/* afs_read_super
- * read the "super block" for AFS - roughly eguivalent to struct vfs.
- * dev, covered, s_rd_only, s_dirt, and s_type will be set by read_super.
- */
 #if defined(AFS_LINUX26_ENV)
-#ifdef GET_SB_HAS_STRUCT_VFSMOUNT
-int
-afs_get_sb(struct file_system_type *fs_type, int flags,
-	   const char *dev_name, void *data, struct vfsmount *mnt)
-#else
-static struct superblock *
-afs_get_sb(struct file_system_type *fs_type, int flags,
-	   const char *dev_name, void *data)
-#endif
-{
-#ifdef GET_SB_HAS_STRUCT_VFSMOUNT
-    return get_sb_nodev(fs_type, flags, data, afs_fill_super, mnt);
-#else
-    return get_sb_nodev(fs_type, flags, data, afs_fill_super);
-#endif
-}
-
+struct backing_dev_info afs_backing_dev_info = {
+	.ra_pages	= (VM_MAX_READAHEAD * 1024) / PAGE_CACHE_SIZE,
+	.state		= 0,
+};
 
 struct file_system_type afs_fs_type = {
     .owner = THIS_MODULE,
@@ -101,10 +83,16 @@ struct file_system_type afs_fs_type = {
 };
 #endif
 
+/* afs_read_super
+ * read the "super block" for AFS - roughly eguivalent to struct vfs.
+ * dev, covered, s_rd_only, s_dirt, and s_type will be set by read_super.
+ */
 #if defined(AFS_LINUX26_ENV)
-struct backing_dev_info afs_backing_dev_info = {
-    .ra_pages		= 0, /* disable readahead, afs does prefetch */
-};
+static struct super_block *
+afs_get_sb(struct file_system_type *fs_type, int flags, const char *dev_name, void *data)
+{
+    return get_sb_nodev(fs_type, flags, data, afs_fill_super);
+}
 
 int
 afs_fill_super(struct super_block *sb, void *data, int silent)
@@ -270,11 +258,7 @@ afs_alloc_inode(struct super_block *sb)
 {
     struct vcache *vcp;
 
-#if defined(SLAB_KERNEL)
     vcp = (struct vcache *) kmem_cache_alloc(afs_inode_cachep, SLAB_KERNEL);
-#else
-    vcp = (struct vcache *) kmem_cache_alloc(afs_inode_cachep, GFP_KERNEL);
-#endif
     if (!vcp)
 	return NULL;
 
@@ -316,7 +300,8 @@ afs_init_inodecache(void)
 void
 afs_destroy_inodecache(void)
 {
-    (void) kmem_cache_destroy(afs_inode_cachep);
+    if (kmem_cache_destroy(afs_inode_cachep))
+	printk(KERN_INFO "afs_inode_cache: not all structures were freed\n");
 }
 #else
 int
@@ -347,6 +332,25 @@ afs_clear_inode(struct inode *ip)
 #endif
 }
 
+/* afs_put_inode
+ * Linux version of inactive.  When refcount == 2, we are about to
+ * decrement to 1 and the only reference remaining should be for
+ * the VLRU
+ */
+
+static void
+afs_put_inode(struct inode *ip)
+{
+    struct vcache *vcp = VTOAFS(ip);
+
+    AFS_GLOCK();
+    if (VREFCOUNT(vcp) == 2) {
+	if (VREFCOUNT(vcp) == 2)
+	    afs_InactiveVCache(vcp, NULL);
+    }
+    AFS_GUNLOCK();
+}
+
 /* afs_put_super
  * Called from unmount to release super_block. */
 static void
@@ -389,11 +393,7 @@ afs_put_super(struct super_block *sbp)
  */
 #if defined(AFS_LINUX26_ENV)
 int
-#if defined(STATFS_TAKES_DENTRY)
-afs_statfs(struct dentry *dentry, struct kstatfs *statp)
-#else
 afs_statfs(struct super_block *sbp, struct kstatfs *statp)
-#endif
 #elif defined(AFS_LINUX24_ENV)
 int
 afs_statfs(struct super_block *sbp, struct statfs *statp)
@@ -416,13 +416,8 @@ afs_statfs(struct super_block *sbp, struct statfs *__statp, int size)
 
     AFS_STATCNT(afs_statfs);
 
-    /* hardcode in case that which is giveth is taken away */
-    statp->f_type = 0x5346414F;
-#if defined(STATFS_TAKES_DENTRY)
-    statp->f_bsize = dentry->d_sb->s_blocksize;
-#else
+    statp->f_type = 0;		/* Can we get a real type sometime? */
     statp->f_bsize = sbp->s_blocksize;
-#endif
     statp->f_blocks = statp->f_bfree = statp->f_bavail = statp->f_files =
 	statp->f_ffree = 9000000;
     statp->f_fsid.val[0] = AFS_VFSMAGIC;
@@ -441,6 +436,7 @@ struct super_operations afs_sops = {
   .destroy_inode =	afs_destroy_inode,
 #endif
   .clear_inode =	afs_clear_inode,
+  .put_inode =		afs_put_inode,
   .put_super =		afs_put_super,
   .statfs =		afs_statfs,
 #if !defined(AFS_LINUX24_ENV)
@@ -500,12 +496,7 @@ vattr2inode(struct inode *ip, struct vattr *vp)
     ip->i_ino = vp->va_nodeid;
     ip->i_nlink = vp->va_nlink;
     ip->i_blocks = vp->va_blocks;
-#ifdef STRUCT_INODE_HAS_I_BLKBITS
-    ip->i_blkbits = AFS_BLKBITS;
-#endif
-#ifdef STRUCT_INODE_HAS_I_BLKSIZE
     ip->i_blksize = vp->va_blocksize;
-#endif
     ip->i_rdev = vp->va_rdev;
     ip->i_mode = vp->va_mode;
     ip->i_uid = vp->va_uid;
