@@ -17,7 +17,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/auth/ktc.c,v 1.15.2.11 2009/03/20 22:45:37 shadow Exp $");
+    ("$Header: /cvs/openafs/src/auth/ktc.c,v 1.16 2004/08/19 01:07:40 kolya Exp $");
 
 #if defined(UKERNEL)
 #include "afs/sysincludes.h"
@@ -40,7 +40,6 @@ RCSID
 #include <unistd.h>
 #endif
 #include <stdio.h>
-#include <stdlib.h>
 #include <afs/stds.h>
 #include <afs/pthread_glock.h>
 #include <sys/types.h>
@@ -50,16 +49,16 @@ RCSID
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
+#ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
 #include <afs/vice.h>
 #ifdef	AFS_AIX_ENV
 #include <sys/lockf.h>
-#ifdef AFS_AIX51_ENV
-#include <sys/cred.h>
-#ifdef HAVE_SYS_PAG_H
-#include <sys/pag.h>
-#endif
-#endif
 #endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -72,7 +71,7 @@ RCSID
 
 /* For malloc() */
 #include <stdlib.h>
-#include "ktc.h"
+
 
 #ifdef	notdef
 /* AFS_KERBEROS_ENV is now conditionally defined in the Makefile */
@@ -129,16 +128,11 @@ static struct flock fileUlock = { F_UNLCK, 0, 0, 0, 0 };
 /* the following routines aren't static anymore on behalf of the kerberos IV
  * compatibility library built in subtree krb.
  */
-int afs_tf_init(char *, int);
-int afs_tf_get_pname(char *);
-int afs_tf_get_pinst(char *);
-int afs_tf_get_cred(struct ktc_principal *, struct ktc_token *);
-int afs_tf_save_cred(struct ktc_principal *, struct ktc_token *, 
-		     struct ktc_principal *);
-int afs_tf_close(void);
-int afs_tf_create(char *, char *);
-int afs_tf_dest_tkt(void);
-static void ktc_LocalCell(void);
+int afs_tf_init(), afs_tf_get_pname(), afs_tf_get_pinst(), afs_tf_get_cred();
+int afs_tf_save_cred(), afs_tf_close(), afs_tf_create();
+int afs_tf_dest_tkt();
+static void ktc_LocalCell();
+char *ktc_tkt_string();
 #endif /* AFS_KERBEROS_ENV */
 
 #ifdef AFS_DUX40_ENV
@@ -258,10 +252,11 @@ static struct {
  * now. */
 
 static int
-NewSetToken(struct ktc_principal *aserver, 
-	    struct ktc_token *atoken, 
-	    struct ktc_principal *aclient, 
-            afs_int32 flags)
+NewSetToken(aserver, atoken, aclient, flags)
+     struct ktc_principal *aserver;
+     struct ktc_principal *aclient;
+     struct ktc_token *atoken;
+     afs_int32 flags;
 {
     TRY_KERNEL(KTC_SETTOKEN_OP, aserver, aclient, atoken,
 	       sizeof(struct ktc_token));
@@ -273,8 +268,10 @@ NewSetToken(struct ktc_principal *aserver,
 (3*sizeof(afs_int32)+MAXKTCTICKETLEN+sizeof(struct ClearToken)+MAXKTCREALMLEN)
 
 static int
-OldSetToken(struct ktc_principal *aserver, struct ktc_token *atoken, 
-	    struct ktc_principal *aclient, afs_int32 flags)
+OldSetToken(aserver, atoken, aclient, flags)
+     struct ktc_principal *aserver, *aclient;
+     struct ktc_token *atoken;
+     afs_int32 flags;
 {
     struct ViceIoctl iob;
     char tbuffer[MAXPIOCTLTOKENLEN];
@@ -418,11 +415,12 @@ OldSetToken(struct ktc_principal *aserver, struct ktc_token *atoken,
     return 0;
 }
 
-int
-ktc_SetToken(struct ktc_principal *aserver,
-    struct ktc_token *atoken,
-    struct ktc_principal *aclient,
-    afs_int32 flags)
+
+ktc_SetToken(aserver, atoken, aclient, flags)
+     struct ktc_principal *aserver;
+     struct ktc_principal *aclient;
+     struct ktc_token *atoken;
+     afs_int32 flags;
 {
     int ncode, ocode;
 
@@ -490,9 +488,10 @@ ktc_SetToken(struct ktc_principal *aserver,
 /* get token, given server we need and token buffer.  aclient will eventually
  * be set to our identity to the server.
  */
-int
-ktc_GetToken(struct ktc_principal *aserver, struct ktc_token *atoken, 
-	     int atokenLen, struct ktc_principal *aclient)
+ktc_GetToken(aserver, atoken, atokenLen, aclient)
+     struct ktc_principal *aserver, *aclient;
+     int atokenLen;
+     struct ktc_token *atoken;
 {
     struct ViceIoctl iob;
     char tbuffer[MAXPIOCTLTOKENLEN];
@@ -504,9 +503,7 @@ ktc_GetToken(struct ktc_principal *aserver, struct ktc_token *atoken,
     afs_int32 temp;
     int maxLen;			/* biggest ticket we can copy */
     int tktLen;			/* server ticket length */
-#ifdef AFS_KERBEROS_ENV
     char found = 0;
-#endif
 
     LOCK_GLOBAL_MUTEX;
 #ifndef NO_AFS_CLIENT
@@ -544,8 +541,8 @@ ktc_GetToken(struct ktc_principal *aserver, struct ktc_token *atoken,
 		    found = 1;
 	    } else {
 		char tmpstring[MAXHOSTCHARS];
-		afs_tf_get_pname(tmpstring);
-		afs_tf_get_pinst(tmpstring);
+		afs_tf_get_pname(&tmpstring);
+		afs_tf_get_pinst(&tmpstring);
 		found = 1;
 	    }
 	}
@@ -675,8 +672,8 @@ ktc_GetToken(struct ktc_principal *aserver, struct ktc_token *atoken,
  * NOT IMPLEMENTED YET!
  */
 #ifndef NO_AFS_CLIENT
-int
-ktc_ForgetToken(struct ktc_principal *aserver)
+ktc_ForgetToken(aserver)
+     struct ktc_principal *aserver;
 {
     int rc;
 
@@ -693,10 +690,9 @@ ktc_ForgetToken(struct ktc_principal *aserver)
  * next rock in (*aindex).  (*aserver) is set to the relevant ticket on
  * success.  */
 
-int
-ktc_ListTokens(int aprevIndex,
-    int *aindex,
-    struct ktc_principal *aserver)
+ktc_ListTokens(aprevIndex, aindex, aserver)
+     int aprevIndex, *aindex;
+     struct ktc_principal *aserver;
 {
     struct ViceIoctl iob;
     char tbuffer[MAXPIOCTLTOKENLEN];
@@ -846,7 +842,7 @@ ktc_ListTokens(int aprevIndex,
 /* discard all tokens from this user's cache */
 
 static int
-NewForgetAll(void)
+NewForgetAll()
 {
 #ifndef NO_AFS_CLIENT
     TRY_KERNEL(KTC_FORGETALLTOKENS_OP, 0, 0, 0, 0);
@@ -855,7 +851,7 @@ NewForgetAll(void)
 }
 
 static int
-OldForgetAll(void)
+OldForgetAll()
 {
     struct ViceIoctl iob;
     register afs_int32 code;
@@ -877,7 +873,7 @@ OldForgetAll(void)
 }
 
 int
-ktc_ForgetAllTokens(void)
+ktc_ForgetAllTokens()
 {
     int ncode, ocode;
 
@@ -905,8 +901,7 @@ ktc_ForgetAllTokens(void)
 /* ktc_OldPioctl - returns a boolean true if the kernel supports only the old
  * pioctl interface for delivering AFS tickets to the cache manager. */
 
-int
-ktc_OldPioctl(void)
+ktc_OldPioctl()
 {
     int rc;
     LOCK_GLOBAL_MUTEX;
@@ -956,13 +951,12 @@ ktc_OldPioctl(void)
  *	c. In tf_close, be sure it gets reinitialized to a negative
  *	   number. 
  */
-static int fd = -1;
-static int curpos;			/* Position in tfbfr */
-static int lastpos;			/* End of tfbfr */
+static fd = -1;
+static curpos;			/* Position in tfbfr */
+static lastpos;			/* End of tfbfr */
 static char tfbfr[BUFSIZ];	/* Buffer for ticket data */
 
-static int tf_gets(char *, int);
-static int tf_read(char *, int);
+static tf_gets(), tf_read();
 
 /*
  * This file contains routines for manipulating the ticket cache file.
@@ -1027,8 +1021,8 @@ static int tf_read(char *, int);
  * TKT_FIL_LCK  - couldn't lock the file, even after a retry
  */
 
-int
-afs_tf_init(char *tf_name, int rw)
+afs_tf_init(tf_name, rw)
+     char *tf_name;
 {
     int wflag;
     int me;
@@ -1121,8 +1115,8 @@ afs_tf_init(char *tf_name, int rw)
  * was longer than MAXKTCNAMELEN, TKT_FIL_FMT is returned. 
  */
 
-int
-afs_tf_get_pname(char *p)
+afs_tf_get_pname(p)
+     char *p;
 {
     if (fd < 0) {
 	return TKT_FIL_INI;
@@ -1142,8 +1136,8 @@ afs_tf_get_pname(char *p)
  * instance may be null. 
  */
 
-int
-afs_tf_get_pinst(char *inst)
+afs_tf_get_pinst(inst)
+     char *inst;
 {
     if (fd < 0) {
 	return TKT_FIL_INI;
@@ -1164,12 +1158,12 @@ afs_tf_get_pinst(char *inst)
  * EOF          - end of file encountered
  */
 
-int
-afs_tf_get_cred(struct ktc_principal *principal, struct ktc_token *token)
+afs_tf_get_cred(principal, token)
+     struct ktc_principal *principal;
+     struct ktc_token *token;
 {
     int k_errno;
     int kvno, lifetime;
-    long mit_compat;		/* MIT Kerberos 5 with Krb4 uses a "long" for issue_date */
 
     if (fd < 0) {
 	return TKT_FIL_INI;
@@ -1206,10 +1200,10 @@ afs_tf_get_cred(struct ktc_principal *principal, struct ktc_token *token)
 	/* don't try to read a silly amount into ticket->dat */
 	token->ticketLen > MAXKTCTICKETLEN
 	|| tf_read((char *)(token->ticket), token->ticketLen) < 1
-	|| tf_read((char *)&mit_compat, sizeof(mit_compat)) < 1) {
+	|| tf_read((char *)&(token->startTime),
+		   sizeof(token->startTime)) < 1) {
 	return TKT_FIL_FMT;
     }
-    token->startTime = mit_compat;
     token->endTime = life_to_time(token->startTime, lifetime);
     token->kvno = kvno;
     return 0;
@@ -1223,8 +1217,7 @@ afs_tf_get_cred(struct ktc_principal *principal, struct ktc_token *token)
  * The return value is not defined.
  */
 
-int
-afs_tf_close(void)
+afs_tf_close()
 {
     if (!(fd < 0)) {
 #if defined(AFS_AIX_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_SUN5_ENV)
@@ -1236,7 +1229,6 @@ afs_tf_close(void)
 	fd = -1;		/* see declaration of fd above */
     }
     memset(tfbfr, 0, sizeof(tfbfr));
-    return 0;
 }
 
 /*
@@ -1257,10 +1249,11 @@ afs_tf_close(void)
  *		file is seriously ill.
  */
 
-static int
-tf_gets(register char *s, int n)
+static
+tf_gets(s, n)
+     register char *s;
 {
-    register int count;
+    register count;
 
     if (fd < 0) {
 	return TKT_FIL_INI;
@@ -1292,10 +1285,12 @@ tf_gets(register char *s, int n)
  * 0		on end of file or read error
  */
 
-static int
-tf_read(register char *s, register int n)
+static
+tf_read(s, n)
+     register char *s;
+     register n;
 {
-    register int count;
+    register count;
 
     for (count = n; count > 0; --count) {
 	if (curpos >= sizeof(tfbfr)) {
@@ -1310,6 +1305,8 @@ tf_read(register char *s, register int n)
     return n;
 }
 
+char *tkt_string();
+
 /*
  * afs_tf_save_cred() appends an incoming ticket to the end of the ticket
  * file.  You must call afs_tf_init() before calling afs_tf_save_cred().
@@ -1321,10 +1318,10 @@ tf_read(register char *s, register int n)
  * called previously, and KFAILURE for anything else that went wrong.
  */
 
-int
-afs_tf_save_cred(struct ktc_principal *aserver, 
-		 struct ktc_token *atoken, 
-		 struct ktc_principal *aclient)
+afs_tf_save_cred(aserver, atoken, aclient)
+     struct ktc_principal *aserver;
+     struct ktc_principal *aclient;
+     struct ktc_token *atoken;	/* Token */
 {
     char realm[MAXKTCREALMLEN + 1];
     char junk[MAXKTCNAMELEN];
@@ -1334,7 +1331,6 @@ afs_tf_save_cred(struct ktc_principal *aserver,
     off_t start;
     int lifetime, kvno;
     int count;			/* count for write */
-    long mit_compat;		/* MIT Kerberos 5 with Krb4 uses a "long" for issue_date */
 
     if (fd < 0) {		/* fd is ticket file as set by afs_tf_init */
 	return TKT_FIL_INI;
@@ -1404,9 +1400,8 @@ afs_tf_save_cred(struct ktc_principal *aserver,
     if (write(fd, atoken->ticket, count) != count)
 	goto bad;
     /* Issue date */
-    mit_compat = atoken->startTime;
-    if (write(fd, (char *)&mit_compat, sizeof(mit_compat))
-	!= sizeof(mit_compat))
+    if (write(fd, (char *)&atoken->startTime, sizeof(afs_int32))
+	!= sizeof(afs_int32))
 	goto bad;
 
     /* Actually, we should check each write for success */
@@ -1423,6 +1418,8 @@ afs_tf_save_cred(struct ktc_principal *aserver,
  * <mit-copyright.h>.
  */
 
+char *getenv();
+
 /*
  * This routine is used to generate the name of the file that holds
  * the user's cache of server tickets and associated session keys.
@@ -1440,26 +1437,20 @@ afs_tf_save_cred(struct ktc_principal *aserver,
 static char krb_ticket_string[4096] = "";
 
 char *
-ktc_tkt_string(void)
-{
-    return ktc_tkt_string_uid(getuid());
-}
-
-char *
-ktc_tkt_string_uid(afs_uint32 uid)
+ktc_tkt_string()
 {
     char *env;
 
     LOCK_GLOBAL_MUTEX;
     if (!*krb_ticket_string) {
-	if ((env = getenv("KRBTKFILE"))) {
+	if (env = getenv("KRBTKFILE")) {
 	    (void)strncpy(krb_ticket_string, env,
 			  sizeof(krb_ticket_string) - 1);
 	    krb_ticket_string[sizeof(krb_ticket_string) - 1] = '\0';
 	} else {
 	    /* 32 bits of signed integer will always fit in 11 characters
 	     * (including the sign), so no need to worry about overflow */
-	    (void)sprintf(krb_ticket_string, "%s%d", TKT_ROOT, uid);
+	    (void)sprintf(krb_ticket_string, "%s%d", TKT_ROOT, getuid());
 	}
     }
     UNLOCK_GLOBAL_MUTEX;
@@ -1478,7 +1469,8 @@ ktc_tkt_string_uid(afs_uint32 uid)
  */
 
 void
-ktc_set_tkt_string(char * val)
+ktc_set_tkt_string(val)
+     char *val;
 {
 
     LOCK_GLOBAL_MUTEX;
@@ -1495,8 +1487,9 @@ ktc_set_tkt_string(char * val)
  * success, or KFAILURE if something goes wrong.
  */
 
-int
-afs_tf_create(char *pname, char *pinst)
+afs_tf_create(pname, pinst)
+     char *pname;
+     char *pinst;
 {
     int tktfile;
     int me, metoo;
@@ -1573,8 +1566,7 @@ afs_tf_create(char *pname, char *pinst)
  * failure.
  */
 
-int
-afs_tf_dest_tkt(void)
+afs_tf_dest_tkt()
 {
     char *file = ktc_tkt_string();
     int i, fd;
@@ -1614,14 +1606,8 @@ afs_tf_dest_tkt(void)
 }
 
 static afs_uint32
-curpag(void)
+curpag()
 {
-#if defined(AFS_AIX51_ENV)
-    int code = getpagvalue("afs");
-    if (code < 0 && errno == EINVAL)
-	code = 0;
-    return code;
-#else
     gid_t groups[NGROUPS_MAX];
     afs_uint32 g0, g1;
     afs_uint32 h, l, ret;
@@ -1645,11 +1631,10 @@ curpag(void)
 	    return -1;
     }
     return -1;
-#endif
 }
 
-int
-ktc_newpag(void)
+
+ktc_newpag()
 {
     extern char **environ;
 
@@ -1668,7 +1653,7 @@ ktc_newpag(void)
     if (pag == -1) {
 	sprintf(fname, "%s%d", prefix, getuid());
     } else {
-	sprintf(fname, "%sp%ld", prefix, (long int) pag);
+	sprintf(fname, "%sp%ld", prefix, pag);
     }
     ktc_set_tkt_string(fname);
 
@@ -1676,7 +1661,7 @@ ktc_newpag(void)
 	numenv++;
     newenv = (char **)malloc((numenv + 2) * sizeof(char *));
 
-    for (senv = environ, denv = newenv; *senv; senv++) {
+    for (senv = environ, denv = newenv; *senv; *senv++) {
 	if (strncmp(*senv, "KRBTKFILE=", 10) != 0)
 	    *denv++ = *senv;
     }
@@ -1687,7 +1672,6 @@ ktc_newpag(void)
     *++denv = 0;
     environ = newenv;
     UNLOCK_GLOBAL_MUTEX;
-    return 0;
 }
 
 /*
@@ -1695,7 +1679,7 @@ ktc_newpag(void)
  * find out what the local cell is.
  */
 static void
-ktc_LocalCell(void)
+ktc_LocalCell()
 {
     int code;
     struct afsconf_dir *conf;
