@@ -22,6 +22,8 @@
 
 #define INCLUDE_RXKAD_PRIVATE_DECLS
 
+RCSID
+    ("$Header: /cvs/openafs/src/rxkad/rxkad_common.c,v 1.29 2006/02/28 00:17:54 shadow Exp $");
 
 #ifdef KERNEL
 #ifndef UKERNEL
@@ -48,7 +50,6 @@
 #else /* KERNEL */
 #include <afs/stds.h>
 #include <sys/types.h>
-#include <string.h>
 #include <time.h>
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -60,7 +61,14 @@
 #endif
 #include <rx/rx.h>
 #include <rx/xdr.h>
-#include <afs/afsutil.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+#endif
+
 #endif /* KERNEL */
 
 #include <des/stats.h>
@@ -303,8 +311,7 @@ FreeObject(struct rx_securityClass *aobj)
     tcp = (struct rxkad_cprivate *)aobj->privateData;
     rxi_Free(aobj, sizeof(struct rx_securityClass));
     if (tcp->type & rxkad_client) {
-	afs_int32 psize = PDATA_SIZE(tcp->ticketLen);
-	rxi_Free(tcp, psize);
+	rxi_Free(tcp, sizeof(struct rxkad_cprivate));
     } else if (tcp->type & rxkad_server) {
 	rxi_Free(tcp, sizeof(struct rxkad_sprivate));
     } else {
