@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/venus/fs.c,v 1.24.2.3 2005/07/11 19:03:42 shadow Exp $");
+    ("$Header: /cvs/openafs/src/venus/fs.c,v 1.30 2006/03/30 17:13:40 rees Exp $");
 
 #include <afs/afs_args.h>
 #include <rx/xdr.h>
@@ -69,7 +69,7 @@ static char tspace[1024];
 static struct ubik_client *uclient;
 
 static int GetClientAddrsCmd(), SetClientAddrsCmd(), FlushMountCmd();
-static int RxStatProcCmd(), RxStatPeerCmd();
+static int RxStatProcCmd(), RxStatPeerCmd(), GetFidCmd();
 
 extern char *hostutil_GetNameByINet();
 extern struct hostent *hostutil_GetHostByName();
@@ -3131,6 +3131,61 @@ GetCryptCmd(struct cmd_syndesc *as, char *arock)
     return 0;
 }
 
+#ifdef AFS_DISCON_ENV
+static char *modenames[] = {
+    "discon",
+    "fetchonly",
+    "partial",
+    "nat",
+    "full",
+    NULL
+};
+
+static afs_int32
+DisconCmd(struct cmd_syndesc *as, char *arock)
+{
+    struct cmd_item *ti;
+    char *modename;
+    int modelen;
+    afs_int32 mode, code;
+    struct ViceIoctl blob;
+
+    blob.in = NULL;
+    blob.in_size = 0;
+
+    ti = as->parms[0].items;
+    if (ti) {
+	modename = ti->data;
+	modelen = strlen(modename);
+	for (mode = 0; modenames[mode] != NULL; mode++)
+	    if (!strncasecmp(modename, modenames[mode], modelen))
+		break;
+	if (modenames[mode] == NULL)
+	    printf("Unknown discon mode \"%s\"\n", modename);
+	else {
+	    memcpy(space, &mode, sizeof mode);
+	    blob.in = space;
+	    blob.in_size = sizeof mode;
+	}
+    }
+
+    blob.out_size = sizeof(mode);
+    blob.out = space;
+    code = pioctl(0, VIOC_DISCON, &blob, 1);
+    if (code)
+	Die(errno, NULL);
+    else {
+	memcpy(&mode, space, sizeof mode);
+	if (mode < sizeof modenames / sizeof (char *))
+	    printf("Discon mode is now \"%s\"\n", modenames[mode]);
+	else
+	    printf("Unknown discon mode %d\n", mode);
+    }
+
+    return 0;
+}
+#endif
+
 #include "AFS_component_version_number.c"
 
 int
@@ -3437,6 +3492,17 @@ defect 3069
     ts = cmd_CreateSyntax("setcbaddr", CallBackRxConnCmd, 0, "configure callback connection address");
     cmd_AddParm(ts, "-addr", CMD_SINGLE, CMD_OPTIONAL, "host name or address");
 
+    /* try to find volume location information */
+    ts = cmd_CreateSyntax("getfid", GetFidCmd, 0,
+			  "get fid for file(s)");
+    cmd_AddParm(ts, "-path", CMD_LIST, CMD_OPTIONAL, "dir/file path");
+
+#ifdef AFS_DISCON_ENV
+    ts = cmd_CreateSyntax("discon", DisconCmd, 0,
+			  "disconnection mode");
+    cmd_AddParm(ts, "-mode", CMD_SINGLE, CMD_OPTIONAL, "nat | full");
+#endif
+
     code = cmd_Dispatch(argc, argv);
     if (rxInitDone)
 	rx_Finalize();
@@ -3794,3 +3860,25 @@ RxStatPeerCmd(struct cmd_syndesc *as, char *arock)
     return 0;
 }
 
+static int
+GetFidCmd(struct cmd_syndesc *as, char *arock)
+{
+    struct ViceIoctl blob;
+    struct cmd_item *ti;
+    for (ti = as->parms[0].items; ti; ti = ti->next) {
+      struct VenusFid vfid;
+      
+      blob.out_size = sizeof(struct VenusFid);
+      blob.out = (char *) &vfid;
+      blob.in_size = 0;
+      
+      if (0 == pioctl(ti->data, VIOCGETFID, &blob, 1)) {
+	printf("File %s (%u.%u.%u) contained in volume %u\n",
+	       ti->data, vfid.Fid.Volume, vfid.Fid.Vnode, vfid.Fid.Unique,
+	       vfid.Fid.Volume);
+      }
+    }
+
+    return 0;
+}
+
